print("Starting compiling H01")
require("lib/lincoln")
print("Lincoln compiled")
require("lib/H01")
print("H01 base compiled")

Globals = {
  hasRanAlready = false,
  isWormanDone = false,
  isHaldricDone = false,
  hasCivilianWalkedAway = false,
  isCibleHit = false,
  isArcheryDone = false,
  victory = false,
  hasPoorWeepingOneRun = false,

  TutorialGetObjectFirst = true,
  EdwardZoneFirst = true,
}

---Events
function Initialize(seed)
  print("initializing H01")
  InitializeLincoln()
  InitH01()
  print("base inicialization done")

  AddBridgeRoomOnReveal(function ()
    UpdateScrollIfShownElseCollected(GetScroll("Scroll_12"), ScrollState.persistent)
  end)
  AddThroneRoomOnReveal(function ()
    AttachScrollToNPC(GetActor("Edward"), GetScroll("Scroll_3"))
    Reveal(GetActor("Edward"))
    UnlockAI(GetActor("Edward"))
  end)
  AddBedroomOnReveal(function ()
    UpdateScrollIfShownElseCollected(GetScroll("Scroll_13"), ScrollState.persistent)
  end)
  AddDungeonTowerOnReveal(function ()
    AssignPatrol(GetActor("Worman"), GetPatrol("Patrol_25"))
  end)
  AddWeaponRoomOnReveal(function ()
    AssignPatrol(GetActor("Epee_Bleu_36"), GetPatrol("Patrol_54"))
  end)
  print("on reveal functions added")
  RevealAnySeenByPC()
  print("interriors seen by PC revealed")

  SetDoorLockedPC(GetDoorScript(DungeonTower.Door.dungeonEntrance), true)
  SetDoorLockedPC(GetDoorScript(MiscDoor.garissonEntrance), true)

  Deactivate(GetItem("Arrows_3"))
  Deactivate(GetItem("Purses_7"))
  Deactivate(GetItem("Money_9"))
  Deactivate(GetItem("Clovers_11"))

  LockAI(GetActor("Poor_Weeping_One"), false)

  SetPersistentProperty(GetRobin(), PersistentProperty.arrows, 0)
  SetPersistentProperty(GetRobin(), PersistentProperty.purses, 0)
  print("initializitaion done")
end

function Timer(seconds)
  if seconds % 15 == 0 and not Globals.isArcheryDone then
    SendMessage(GetActor("Shooting_Archer"), 1)
  end

  HandleFlirtingSwordsman()
  HandlePoorWeepingOneRun()
  HandleHaldricDone()
  HandleCivilianWalkedAway()
  HandleWormanDone()
end

function CheckVictoryCondition(seconds)
  if Globals.victory then
    ChooseVictoryDefeatText(0)
    return 1
  end
end

function PostInitialize()
  AddObjective(0, true)
  StartSequence()
  SequenceDisplayPopupText(0)
  Then()
  SequenceDisplayPopupText(1)
  Then()
  SequenceDisplayPopupText(2)
  EndSequence()
end

--Functions
function HandleFlirtingSwordsman()
  if IsActorOutOfAction(GetActor("Flirting_Swordsman")) and not Globals.hasRanAlready then
    AssignPatrol(GetActor("Flirting_Swordsman"), GetPatrol("Patrol_66"))
    AssignPatrol(GetActor("WomanCivilianPoor_2"), GetPatrol("Patrol_59"))
    SetPatrolShouldRun(GetActor("WomanCivilianPoor_2"), true)
    Globals.hasRanAlready = true
  end
end

function HandlePoorWeepingOneRun()
  if not Globals.hasPoorWeepingOneRun and
      IsActorOutOfAction(GetActor("Epee_Bleu_25")) and
      IsActorOutOfAction(GetActor("Epee_Bleu_26")) and
      IsActorOutOfAction(GetActor("Lancier_Bleu_27")) and
      IsActorOutOfAction(GetActor("Lancier_Bleu_28")) then
    SendMessage(GetActor("Poor_Weeping_One"), 1)
  end
end

function HandleWormanDone()
  if not Globals.isWormanDone and IsBonusItemPickedUp(GetItem("Money_6")) then
    Globals.isWormanDone = true
    CompleteObjective(3)
    UpdateScrollIfShownElseCollected(GetScroll("Scroll_10"), ScrollState.persistent)
  end
end

function HandleHaldricDone()
  if not Globals.isHaldricDone and GetPersistentProperty(GetActor("Haldric"), PersistentProperty.money) == 0 then
    Globals.isHaldricDone = true
    CompleteObjective(4)
    UpdateScrollIfShownElseCollected(GetScroll("Scroll_11"), ScrollState.persistent)
  end
end

function HandleCivilianWalkedAway()
  if not Globals.hasCivilianWalkedAway and
      (IsActorOutOfAction(GetActor("Lancier_Bleu_19")) or
        IsActorOutOfAction(GetActor("Lancier_Bleu_20")) or
        IsActorOutOfAction(GetActor("Lancier_Bleu_21"))) then
    AssignPatrol(GetActor("WomanCivilianPoor_7"), GetPatrol("Patrol_58"))
    Globals.hasCivilianWalkedAway = true
  end
end

ArchersDone = function()
  if not Globals.isArcheryDone then
    CompleteObjective(5)
    Globals.isArcheryDone = true
    UpdateScrollState(GetScroll("Scroll_1"), ScrollState.persistent)
    AssignSoldiersToOfficer()
    AssignPatrolPaths()
    HandleOfficerState()
  end
end

function AssignSoldiersToOfficer()
  local subordinates = {
    GetActor("Archer_Bleu_12"),
    GetActor("Archer_Bleu_13"),
    GetActor("Shooting_Archer"),
    GetActor("Epee_Bleu_16"),
    GetActor("Epee_Bleu_17"),
    GetActor("Epee_Bleu_18")
  }

  for _, subordinate in pairs(subordinates) do
    AddAsSquadMember(GetActor("Archer_Officer"), subordinate)
  end
end

function AssignPatrolPaths()
  local patrolPaths = {
    { actor = GetActor("Lancier_Bleu_19"),     patrol = "Patrol_51" },
    { actor = GetActor("Lancier_Bleu_20"),     patrol = "Patrol_52" },
    { actor = GetActor("Lancier_Bleu_21"),     patrol = "Patrol_53" },
    { actor = GetActor("Hallebardier_Bleu_6"), patrol = "Patrol_3" }
  }

  for _, path in pairs(patrolPaths) do
    AssignPatrol(path.actor, GetPatrol(path.patrol))
  end
end

function HandleOfficerState()
  if not IsActorOutOfAction(GetActor("Archer_Officer")) then
    DisplayPopupText(6)
    AssignPatrol(GetActor("Archer_Officer"), GetPatrol("Patrol_39"))
  else
    AssignSecondaryPaths()
  end
end

function AssignSecondaryPaths()
  local secondaryPaths = {
    { actor = GetActor("Archer_Bleu_12"), patrol = "Patrol_72" },
    { actor = GetActor("Archer_Bleu_13"), patrol = "Patrol_71" },
    { actor = GetActor("Shooting_Archer"),     patrol = "Patrol_70" },
    { actor = GetActor("Epee_Bleu_16"),   patrol = "Patrol_69" },
    { actor = GetActor("Epee_Bleu_17"),   patrol = "Patrol_67" },
    { actor = GetActor("Epee_Bleu_18"),   patrol = "Patrol_68" }
  }

  for _, path in pairs(secondaryPaths) do
    AssignPatrol(path.actor, GetPatrol(path.patrol))
  end
end

function MoveSoldiersInside(soldier)
  AssignPatrol(soldier, GetPatrol("Patrol_49"))
  StartSequence()
  SequenceLockAI(soldier)
  Then()
  SequenceMoveIntoBuilding(soldier, GetLocation("Node_1"), MovementStyle.walking)
  EndSequence()
end

SoldiersAlerted = function()
  Globals.isArcheryDone = true
  UpdateScrollState(GetScroll("Scroll_1"), ScrollState.persistent)
  AssignSoldiersToOfficer()
  AssignPatrolPaths()
end

---Actors
PoorWeepingOne_80000344 = {
  ProcessMessage = function(message, unk2, unk3)
    if message == 1 then
      AssignPatrol(GetActor("Epee_Bleu_25"), GetPatrol("Patrol_26"))
      AssignPatrol(GetActor("Epee_Bleu_26"), GetPatrol("Patrol_27"))
      AssignPatrol(GetActor("Lancier_Bleu_27"), GetPatrol("Patrol_28"))
      AssignPatrol(GetActor("Lancier_Bleu_28"), GetPatrol("Patrol_29"))
      Globals.hasPoorWeepingOneRun = true
      StartSequence()
      SequenceMove(GetActor("Poor_Weeping_One"), GetLocation("Node_8"), MovementStyle.running)
      Then()
      SequenceTurnTo(GetActor("Poor_Weeping_One"), GetLocation("Node_9"))
      Then()
      SequenceCall(function()
        AttachScrollToNPC(GetActor("Poor_Weeping_One"), GetScroll("Scroll_15"))
      end)
      EndSequence()
    end
  end
}

Acher03_80000042 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end
}

Acher02_80000041 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end
}

Acher01_80000040 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end,

  ProcessMessage = function(message, unk2, unk3)
    if message == 1 and not Globals.isArcheryDone then
      StartSequence()
      SequenceSendMessage(GetActor("Archer_Officer"), 1)
      Then()
      SequenceTimer(12)
      Then()
      SequenceAction(GetActor("Shooting_Archer"), ActorAction.shoot, GetActorIndex(GetActor("Cible01_2")))
      Then()
      SequenceTimer(12)
      Then()
      SequenceSendMessage(GetActor("Archer_Officer"), 2)
      EndSequence()
    end
  end
}

SergentArchers_80000043 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end,

  ProcessMessage = function(message, unk2, unk3)
    if message == 1 and not Globals.isArcheryDone then
      StartSequence()
      SequenceTurnTo(GetActor("Archer_Officer"), GetLocation("Node_2"))
      EndSequence()
    end

    if message == 2 and not Globals.isArcheryDone then
      SetPersistentProperty(GetActor("Shooting_Archer"), PersistentProperty.arrows,
        (GetPersistentProperty(GetActor("Shooting_Archer"), PersistentProperty.arrows) + 1))
      StartSequence()
      SequenceTurnTo(GetActor("Archer_Officer"), GetActorLocation(GetActor("Shooting_Archer")))
      Then()
      SequencePlayAnim(GetActor("Archer_Officer"), AnimationAction.gatheringSoldiers)
      EndSequence()
    end
  end
}

SoldatCibles01_80000044 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end
}

SoldatCibles02_8000004a = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end
}

SoldatCibles03_80000050 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end
}

LancierEscalierHcour01_8000005f = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end
}

LancierHCour02_80000060 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end
}

BouclierXT01_800000d6 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      AssignPatrol(GetActor("Flirting_Swordsman"), GetPatrol("Patrol_66"))
      AssignPatrol(GetActor("WomanCivilianPoor_2"), GetPatrol("Patrol_59"))
    end
  end
}

---Targets
Cible1_800001bb = {
  ActivatedByArrow = function(actorSource)
    HandleCibleHit(actorSource, GetActor("Cible00_1"))
  end
}

Cible2_800001bd = {
  ActivatedByArrow = function(actorSource)
    HandleCibleHit(actorSource, GetActor("Cible01_2"))
  end
}

Cible3_800001be = {
  ActivatedByArrow = function(actorSource)
    HandleCibleHit(actorSource, GetActor("Cible02_3"))
  end
}

Cible4_800001bf = {
  ActivatedByArrow = function(actorSource)
    HandleCibleHit(actorSource, GetActor("Cible03_4"))
  end
}

function HandleCibleHit(actorSource, cibleActor)
  if not Globals.isCibleHit and IsActorPC(actorSource) then
    Globals.isCibleHit = true
    StartSequence()
    SequencePlayAnimFreeze(cibleActor, AnimationAction.targetHit)
    Then()
    SequenceTimer(10)
    Then()
    SequenceCall(ArchersDone)
    EndSequence()
  end
end

--WayPoints
OfficierdesArchers__6___80000184 = {
  ReachPoint = function(actor)
    StartSequence()
    SequenceCall(function() MoveSoldiersInside(GetActor("Archer_Officer")) end)
    SequenceTimer(12)
    Then()
    SequenceCall(function() MoveSoldiersInside(GetActor("Archer_Bleu_12")) end)
    SequenceTimer(12)
    Then()
    SequenceCall(function() MoveSoldiersInside(GetActor("Archer_Bleu_13")) end)
    SequenceTimer(12)
    Then()
    SequenceCall(function() MoveSoldiersInside(GetActor("Shooting_Archer")) end)
    SequenceTimer(12)
    Then()
    SequenceCall(function() MoveSoldiersInside(GetActor("Epee_Bleu_16")) end)
    SequenceTimer(12)
    Then()
    SequenceCall(function() MoveSoldiersInside(GetActor("Epee_Bleu_17")) end)
    SequenceTimer(12)
    Then()
    SequenceCall(function() MoveSoldiersInside(GetActor("Epee_Bleu_18")) end)
    EndSequence()
  end
}

GirlWay01__1___800001f8 = {
  ReachPoint = function(actor)
    SetPatrolShouldRun(GetActor("Flirting_Swordsman"), true)
    AssignPatrol(GetActor("Flirting_Swordsman"), GetPatrol("Patrol_61"))
    AssignPatrol(actor, GetPatrol("Patrol_65"))
  end
}

GirlWay02__1___800001f9 = {
  ReachPoint = function(actor)
    SetPatrolShouldRun(GetActor("Flirting_Swordsman"), false)
    AssignPatrol(GetActor("Flirting_Swordsman"), GetPatrol("Patrol_60"))
    AssignPatrol(actor, GetPatrol("Patrol_64"))
    StareActor(GetActor("Flirting_Swordsman"), GetActorScript(MiscAnimation.riverPier))
  end
}

--Scolls
ParchArcherDebut_80000258 = {
  Taken = function(actorSource)
    if not Globals.isArcheryDone then
      AddObjective(5, false)
      StartSequence()
      SequenceSetZoomLevel(ZoomLevel.default)
      Then()
      SequenceLockUser()
      SequenceFreezeAll(true)
      Then()
      SequenceScrollCameraTo(GetActorLocation(GetActor("Archer_Officer")))
      Then()
      SequenceTimer(15)
      Then()
      SequenceSpeak(GetActor("Archer_Officer"), 61)
      Then()
      SequenceDisplayPopupText(5)
      Then()
      SequenceDisplayPopupText(22)
      Then()
      SequenceSendMessage(GetActor("Shooting_Archer"), 1)
      Then()
      SequenceFreezeAll(false)
      SequenceUnLockUser()
      EndSequence()
    end
  end
}

ParchArgent_8000047f = {
  Taken = function(actorSource)
    if Globals.isArcheryDone then
      DisplayPopupText(19)
    end
    Activate(GetItem("Money_9"))
    UpdateScrollState(ThisScroll(), ScrollState.collected)
  end
}

ParchEdwardSpeak_8000025a = {
  Taken = function(actorSource)
    AddObjective(1, true)
    CompleteObjective(0)
    Reveal(GetActor("ManCivilianFriend_5"))
    StartSequence()
    SequenceSetZoomLevel(ZoomLevel.default)
    Then()
    SequenceLockUser()
    SequenceFreezeAll(true)
    Then()
    SequenceDisplayPopupText(8)
    Then()
    SequenceDisplayPopupText(9)
    Then()
    SequenceDisplayPopupText(10)
    Then()
    SequenceScrollCameraTo(GetLocation("Node_10"))
    Then()
    SequenceFreezeAll(false)
    Then()
    SequenceMove(GetActor("ManCivilianFriend_5"), GetLocation("Node_10"), MovementStyle.walking)
    Then()
    SequenceCall(function()
      AssignPost(GetActor("ManCivilianFriend_5"), GetLocation("Node_10"), ActorDirection.south)
      AttachScrollToNPC(GetActor("ManCivilianFriend_5"), GetScroll("Scroll_4"))
    end)
    Then()
    SequenceFreezeAll(true)
    Then()
    SequenceTimer(40)
    Then()
    SequenceJumpCameraTo(GetActorLocation(GetRobin()))
    SequenceFreezeAll(false)
    SequenceUnLockUser()
    EndSequence()
    RemoveScrollFromNPC(GetActor("Edward"))
    UpdateScrollState(ThisScroll(), ScrollState.collected)
  end
}

ParchFilsEdward_8000025b = {
  Taken = function(actorSource)
    AddObjective(2, true)
    CompleteObjective(1)
    DisplayPopupText(11)
    Globals.victory = true
    RemoveScrollFromNPC(GetActor("ManCivilianFriend_5"))
    UpdateScrollState(ThisScroll(), ScrollState.collected)
  end
}

TutGrimper_8000025d = {
  Taken = function(actorSource)
    StartSequence()
    SequenceSetZoomLevel(ZoomLevel.default)
    Then()
    SequenceLockUser()
    Then()
    SequenceFreezeAll(true)
    Then()
    SequenceScrollCameraTo(GetLocation("Node_12"))
    Then()
    SequenceTimer(12)
    Then()
    SequenceDisplayPopupText(12)
    Then()
    SequenceTimer(12)
    Then()
    SequenceFreezeAll(false)
    SequenceUnLockUser()
    EndSequence()
  end
}

TutJump_8000025e = {
  Taken = function(actorSource)
    DisplayPopupText(13)
  end
}

TutKO_8000025f = {
  Taken = function(actorSource)
    DisplayPopupText(14)
    DisplayPopupText(21)
  end
}

TutBeggar_80000261 = {
  Taken = function(actorSource)
    StartSequence()
    SequenceSetZoomLevel(1.0)
    Then()
    SequenceLockUser()
    Then()
    SequenceScrollCameraTo(GetLocation("Node_1"))
    Then()
    SequenceTimer(15)
    Then()
    SequenceReveal(GetActor("Mendicant_6"))
    Then()
    SequenceDisplayPopupText(16)
    Then()
    SequenceTimer(40)
    SequenceJumpCameraTo(GetActorLocation(actorSource))
    Then()
    SequenceUnLockUser()
    EndSequence()
  end
}

TutGetObject_80000262 = {
  Taken = function(actorSource)
    DisplayPopupText(15)
    if Globals.TutorialGetObjectFirst then
      Globals.TutorialGetObjectFirst = false
      DisplayPopupText(17)
      Activate(GetItem("Purses_7"))
    end
  end
}

BeggarWorman_80000277 = {
  Taken = function(actorSource)
    if not IsBonusItemPickedUp(GetItem("Money_6")) then
      AddObjective(3, false)
      StartSequence()
      SequenceSetZoomLevel(ZoomLevel.default)
      Then()
      SequenceLockUser()
      SequenceFreezeAll(true)
      Then()
      SequenceScrollCameraTo(GetLocation("Node_4"))
      Then()
      SequenceCall(RevealDungeonTower)
      Then()
      SequenceReveal(GetActor("Worman"))
      SequenceReveal(GetActor("Arbaletrier_Bleu_23"))
      Then()
      SequenceTimer(15)
      Then()
      SequenceDisplayPopupText(4)
      Then()
      SequenceTimer(40)
      Then()
      SequenceJumpCameraTo(GetActorLocation(actorSource))
      Then()
      SequenceUnLockUser()
      SequenceFreezeAll(false)
      EndSequence()
    end
    UpdateScrollState(ThisScroll(), ScrollState.collected)
  end
}

BeggarHaldric_80000278 = {
  Taken = function(actorSource)
    if not Globals.isHaldricDone then
      AddObjective(4, false)
      StartSequence()
      SequenceSetZoomLevel(ZoomLevel.default)
      Then()
      SequenceLockUser()
      SequenceFreezeAll(true)
      Then()
      SequenceDisplayPopupText(7)
      Then()
      SequenceScrollCameraTo(GetActorLocation(GetActor("Haldric")))
      Then()
      SequenceTimer(15)
      Then()
      SequenceReveal(GetActor("Haldric"))
      Then()
      SequenceTimer(40)
      Then()
      SequenceJumpCameraTo(GetActorLocation(actorSource))
      Then()
      SequenceFreezeAll(false)
      SequenceUnLockUser()
      EndSequence()
    end
    UpdateScrollState(ThisScroll(), ScrollState.collected)
  end
}

Beggarpatch01_800002c4 = {
  Taken = function(actorSource)
    RevealBridgeRoom()
    StartSequence()
    SequenceSetZoomLevel(ZoomLevel.default)
    Then()
    SequenceLockUser()
    SequenceFreezeAll(true)
    Then()
    SequenceScrollCameraTo(GetLocation("Node_14"))
    Then()
    SequenceTimer(15)
    Then()
    SequenceDisplayPopupText(20)
    Then()
    SequenceTimer(40)
    Then()
    SequenceJumpCameraTo(GetActorLocation(actorSource))
    Then()
    SequenceFreezeAll(false)
    SequenceUnLockUser()
    EndSequence()
  end
}

TutOpenSalle_8000046d = {
  Taken = function(actorSource)
    if not IsPatchApplied(GetPatchScript(7)) then
      StartSequence()
      SequenceSetZoomLevel(ZoomLevel.default)
      Then()
      SequenceLockUser()
      SequenceFreezeAll(true)
      Then()
      SequenceScrollCameraTo(GetLocation("Node_13"))
      Then()
      SequenceTimer(15)
      Then()
      SequenceCall(RevealBedRoom)
      Then()
      SequenceTimer(40)
      Then()
      SequenceJumpCameraTo(GetActorLocation(actorSource))
      Then()
      SequenceFreezeAll(false)
      SequenceUnLockUser()
      EndSequence()
    end
    UpdateScrollState(ThisScroll(), ScrollState.collected)
  end
}

ArrowsAtEndSequence_800006e3 = {
  Taken = function(actorSource)
    Activate(GetItem("Arrows_3"))
    UpdateScrollState(ThisScroll(), ScrollState.collected)
  end
}

PoorManScroll_80000349 = {
  Taken = function(actorSource)
    Activate(GetItem("Clovers_11"))
    RemoveScrollFromNPC(GetActor("Poor_Weeping_One"))
    UpdateScrollState(ThisScroll(), ScrollState.collected)
    StartSequence()
    SequenceSpeak(GetActor("Poor_Weeping_One"), 86)
    Then()
    SequenceDisplayPopupText(3)
    EndSequence()
  end
}


---Zones
EdwardZone_80000275 = {
  Enter = function(actor)
    if IsActorPC(actor) and Globals.EdwardZoneFirst then
      StopActor(actor)
      Globals.EdwardZoneFirst = false
      StartSequence()
      SequenceSetZoomLevel(ZoomLevel.default)
      Then()
      SequenceLockUser()
      SequenceFreezeAll(true)
      Then()
      SequenceScrollCameraTo(GetLocation("Node_5"))
      Then()
      SequenceTimer(15)
      Then()
      SequenceDisplayPopupText(18)
      Then()
      SequenceTimer(40)
      Then()
      SequenceUnLockUser()
      SequenceFreezeAll(false)
      EndSequence()
    end
  end
}

persecutionZone_80000346 = {
  Exit = function(actor)
    if GetNumberOfActorsInSector(GetLocation("Node_27")) == 1 then
      if not Globals.hasPoorWeepingOneRun then
        SendMessage(GetActor("Poor_Weeping_One"), 1)
      end
    end
  end
}
