require("enums")

function RevealAll()
  for _, actor in pairs(GetAllActors()) do
    Reveal(actor)
  end
end

function RevealNPCsInSector(location)
  local numberOfActors = GetNumberOfActorsInSector(location)

  for i = 0, numberOfActors - 1 do
    local actor = GetActorInSector(location, i)

    if IsActorNPC(actor) then
      Reveal(actor)
    end
  end
end

function RemoveScrollFromNPC(actor)
  if IsActorNPC(actor) then
    AttachScrollToNPC(actor, 0)
  end
end

function UpdateScrollIfShownElseCollected(scroll, ifShown)
  if GetScrollState(scroll) == ScrollState.hidden then
    UpdateScrollState(scroll, ScrollState.collected)
  else
    UpdateScrollState(scroll, ifShown)
  end
end

function UpdateScrollState(scroll, state)
  local oldState = GetScrollState(scroll)
  if oldState < state then
    if oldState <= ScrollState.default then
      SetScrollState(scroll, state)
    end
  else
    if oldState == ScrollState.persistent and state == ScrollState.collected then
      SetScrollState(scroll, state)
    end
  end
end

function ActivateActors(actors)
  for _, actor in pairs(actors) do
    Activate(actor)
    UnlockAI(actor)
  end
end

function DeactivateActors(actors)
  for _, actor in pairs(actors) do
    Deactivate(actor)
    LockAI(actor, false)
  end
end

function ActivateItems(items)
  for _, item in pairs(items) do
    Activate(item)
  end
end

function DeactivateItems(items)
  for _, item in pairs(items) do
    Deactivate(item)
  end
end

function DeactivateScrolls(scrolls)
  for _, scroll in pairs(scrolls) do
    SetScrollState(scroll, ScrollState.hidden)
  end
end

function ActivateScrolls(scrolls)
  for _, scroll in pairs(scrolls) do
    SetScrollState(scroll, ScrollState.default)
  end
end

function ActivateSherwoodSigns()
  for name, actor in pairs(GetAllActors()) do
    if name:find("Panneau_") then
      Activate(actor)
    end
  end
end

function DeactivateSherwoodSigns()
  for name, actor in pairs(GetAllActors()) do
    if name:find("Panneau_") then
      Deactivate(actor)
    end
  end
end

function PrintAllActors()
  for i = 0, GetNumberOfActorsInEngine() do
    print(i .. " : " .. GetActorName(GetActorScript(i)))
  end
end

function IsPCSafe(actor, exitLocations)
  for _, location in pairs(exitLocations) do
    if IsInside(actor, GetLocation(location)) then
      return true
    end
  end

  return false
end

local lastKeyPressTime = 0
local debounceDelay = 0.2

-- Useful for the KeyPressed event to prevent spamming
-- Return if this equals false
function IsDebounced()
  local currentTime = os.clock()

  if currentTime - lastKeyPressTime < debounceDelay then
    return false
  end

  lastKeyPressTime = currentTime

  return true
end