require("common")
require("enums")
require("lib/lincoln")

local hidden_spaces = {}

AddBattlementOnReveal = function (f)
  table.insert(hidden_spaces.battlement.OnRevealFuncs, f)
end

AddBedroomOnReveal = function (f)
  table.insert(hidden_spaces.bedroom.OnRevealFuncs, f)
end

AddBridgeRoomOnReveal = function (f)
  table.insert(hidden_spaces.bridge_room.OnRevealFuncs, f)
end

AddWeaponRoomOnReveal = function (f)
  table.insert(hidden_spaces.weapon_room.OnRevealFuncs, f)
end

AddThroneRoomOnReveal = function (f)
  table.insert(hidden_spaces.throne_room.OnRevealFuncs, f)
end

AddDungeonTowerOnReveal = function (f)
  table.insert(hidden_spaces.dungeon_tower.OnRevealFuncs, f)
end

AddStaircaseOnReveal = function (f)
  table.insert(hidden_spaces.staircase.OnRevealFuncs, f)
end

AddPrisonOnReveal = function (f)
  table.insert(hidden_spaces.prison.OnRevealFuncs, f)
end

GetObjectsInLocations = function (locations, potentialObjects)
  local result = {}
  for key, a in pairs(potentialObjects) do
    for k2, loc in pairs(locations) do
      if IsInside(a, loc) then
        result[key] = a
      end
    end
  end
  return result
end

Concat = function (table1, table2)
  local result = {}
  for key, value in pairs(table1) do
    result[key] = value
  end
  for key, value in pairs(table2) do
    result[key] = value
  end
  return result
end

local GetAllNPCs = function()
  local NPCs = {}
  for key, a in pairs(GetAllActors()) do
    if IsActorNPC(a) then
      NPCs[key] = a
    end
  end
  return NPCs
end

local GetDeactivatableObjects = function()
  return Concat(GetAllNPCs(), Concat(GetAllItems(), GetAllScrolls()))
end

local SetActive = function(actor, name, isActive)
  if isActive then
    print("Activating " .. name)
    Activate(actor)
    if IsActorNPC(actor) then
      UnlockAI(actor)
    end
  else
    print("Deactivating " .. name)
    Deactivate(actor)
    if IsActorNPC(actor) then
      LockAI(actor, false)
    end
  end
end

local SetActiveIn = function(where, isActive)
  for key, a in pairs(where.Objects) do
    SetActive(a, key, isActive)
  end
end

local AnyPCSees = function(where)
  local i = 0
  while i < GetNumberOfPCs() do
    local potential = GetObjectsInLocations(Concat(where.Interrior, where.Exterrior), {GetPC(i)})
    if #potential > 0 then
      return true
    end
    i = i + 1
  end
  return false
end

local CreateInterriors = function ()
  hidden_spaces = {
    weapon_room = {
      Interrior = {GetLocation("Node_39")},
      Exterrior = {GetLocation("Node_16"), GetLocation("Node_17"), GetLocation("Node_33"), GetLocation("Node_34")},
      RevealFunc = RevealWeaponRoom,
      Objects = {},
      OnRevealFuncs = {}
    }, 
    staircase = {
      Interrior = {GetLocation("Node_40"), GetLocation("Node_41")},
      Exterrior = {GetLocation("Node_23"), GetLocation("Node_24"), GetLocation("Node_25"), GetLocation("Node_32")},
      RevealFunc = RevealStaircase,
      Objects = {},
      OnRevealFuncs = {}
    }, 
    dungeon_tower = {
      Interrior = {GetLocation("Node_35"), GetLocation("Node_45"), GetLocation("Node_46"), 
        GetLocation("Node_47"), GetLocation("Node_48"), GetLocation("Node_49"), },
      Exterrior = {GetLocation("Node_18"), GetLocation("Node_31")},
      RevealFunc = RevealDungeonTower,
      Objects = {},
      OnRevealFuncs = {}
    }, 
    prison = {
      Interrior = {GetLocation("Node_31"), GetLocation("Node_43"), GetLocation("Node_44")},
      Exterrior = {GetLocation("Node_35")},
      RevealFunc = RevealPrison,
      Objects = {},
      OnRevealFuncs = {}
    }, 
    bedroom = {
      Interrior = {GetLocation("Node_37")},
      Exterrior = {GetLocation("Node_21"), GetLocation("Node_22")},
      RevealFunc = RevealBedRoom,
      Objects = {},
      OnRevealFuncs = {}
    }, 
    throne_room = {
      Interrior = {GetLocation("Node_42")},
      Exterrior = {GetLocation("Node_23"), GetLocation("Node_32")},
      RevealFunc = RevealThroneRoom,
      Objects = {},
      OnRevealFuncs = {}
    }, 
    bridge_room = {
      Interrior = {GetLocation("Node_36")},
      Exterrior = {GetLocation("Node_19"), GetLocation("Node_28")},
      RevealFunc = RevealBridgeRoom,
      Objects = {},
      OnRevealFuncs = {}
    }, 
    battlement = {
      Interrior = {GetLocation("Node_38")},
      Exterrior = {GetLocation("Node_20"), GetLocation("Node_29"), GetLocation("Node_30")},
      RevealFunc = RevealBattlement,
      Objects = {},
      OnRevealFuncs = {}
    }, 
  }
end

InitH01 = function ()
  CreateInterriors()

  local all = GetDeactivatableObjects()
  for key, obj in pairs(hidden_spaces) do
    print("location: " .. key)
    obj.Objects = GetObjectsInLocations(obj.Interrior, all)
    SetActiveIn(obj, false)
  end
end

RevealAnySeenByPC = function()
  for key, obj in pairs(hidden_spaces) do
    if AnyPCSees(obj) then
      obj.RevealFunc()
    end
  end
end

RevealWeaponRoom = function()
  if not IsPatchApplied(GetPatchScript(WeaponRoom.patch)) then
    WeaponRoom:Reveal()
    SetActiveIn(hidden_spaces.weapon_room, true)
    for key, f in pairs(hidden_spaces.weapon_room.OnRevealFuncs) do
      f()
    end
  end
end

RevealDungeonTower = function()
  if not IsPatchApplied(GetPatchScript(DungeonTower.patch)) then
    DungeonTower:Reveal()
    SetActiveIn(hidden_spaces.dungeon_tower, true)
    for key, f in pairs(hidden_spaces.dungeon_tower.OnRevealFuncs) do
      f()
    end
  end
end

RevealBedRoom = function()
  if not IsPatchApplied(GetPatchScript(BedRoom.patch)) then
    BedRoom:Reveal()
    SetActiveIn(hidden_spaces.bedroom, true)
    for key, f in pairs(hidden_spaces.bedroom.OnRevealFuncs) do
      f()
    end
  end
end

RevealStaircase = function()
  if not IsPatchApplied(GetPatchScript(StairCase.patch)) then
    StairCase:Reveal()
    SetActiveIn(hidden_spaces.staircase, true)
    for key, f in pairs(hidden_spaces.staircase.OnRevealFuncs) do
      f()
    end
  end
end

RevealThroneRoom = function()
  if not IsPatchApplied(GetPatchScript(ThroneRoom.patch)) then
    ThroneRoom:Reveal()
    SetActiveIn(hidden_spaces.throne_room, true)
    for key, f in pairs(hidden_spaces.throne_room.OnRevealFuncs) do
      f()
    end
  end
end

RevealBridgeRoom = function()
  if not IsPatchApplied(GetPatchScript(DrawBridgeRoom.patch)) then
    DrawBridgeRoom:Reveal()
    SetActiveIn(hidden_spaces.bridge_room, true)
    for key, f in pairs(hidden_spaces.bridge_room.OnRevealFuncs) do
      f()
    end
  end
end

RevealBattlement = function()
  if not IsPatchApplied(GetPatchScript(Battlement.patch)) then
    Battlement:Reveal()
    SetActiveIn(hidden_spaces.battlement, true)
    for key, f in pairs(hidden_spaces.battlement.OnRevealFuncs) do
      f()
    end
  end
end

RevealPrison = function()
  if not IsPatchApplied(GetPatchScript(Dungeon.patch)) then
    Dungeon:Reveal()
    SetActiveIn(hidden_spaces.prison, true)
    for key, f in pairs(hidden_spaces.prison.OnRevealFuncs) do
      f()
    end
  end
end

mecanisme_pont_levis_800001e9 = {
  ActivatedBySword = function(actorSource)
    DrawBridgeRoom:ActivateDrawBridge(GetLocation("Node_15"))
  end
}

DoorUnlockedOrUnlockable = function (index)
  return IsDoorUnlockable(GetDoorScript(index)) or (IsDoorLockedPC(GetDoorScript(index)) == false)
end

--Zones
ZP01BridgeRoom01 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      RevealBridgeRoom()
    end
  end
}

ZP01BridgeRoom02 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      if DoorUnlockedOrUnlockable(53) then
        RevealBridgeRoom()
      end
    end
  end
}

ZP02Prison01 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      if DoorUnlockedOrUnlockable(21) and DoorUnlockedOrUnlockable(20) then
        RevealPrison()
      end
    end
  end
}

ZP03DungeonTower01 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      RevealDungeonTower()
    end
  end
}

ZP03DungeonTower02 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      if DoorUnlockedOrUnlockable(21) and DoorUnlockedOrUnlockable(20) then
        RevealDungeonTower()
      end
    end
  end
}

ZP04Battlement01 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      if DoorUnlockedOrUnlockable(36) and DoorUnlockedOrUnlockable(37) then
        RevealBattlement()
      end
    end
  end
}

ZP04Battlement02 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      if IsDoorLockedPC(GetDoorScript(6)) == false then
        RevealBattlement()
      end
    end
  end
}

ZP04Battlement03 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      if DoorUnlockedOrUnlockable(38) and DoorUnlockedOrUnlockable(37) then
        RevealBattlement()
      end
    end
  end
}

ZP05ThroneAndStairs01 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      RevealStaircase()
      RevealThroneRoom()
    end
  end
}

ZP05ThroneAndStairs02 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      if DoorUnlockedOrUnlockable(0) then
        RevealStaircase()
        RevealThroneRoom()
      end
    end
  end
}

ZP06WeaponRoom01 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      if IsDoorLockedPC(GetDoorScript(4)) == false then
        RevealWeaponRoom()
      end
    end
  end
}

ZP06WeaponRoom02 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      if IsDoorLockedPC(GetDoorScript(5)) == false then
        RevealWeaponRoom()
      end
    end
  end
}

ZP06WeaponRoom03 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      if DoorUnlockedOrUnlockable(26) and DoorUnlockedOrUnlockable(25) then
        RevealWeaponRoom()
      end
    end
  end
}

ZP06WeaponRoom04 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      if DoorUnlockedOrUnlockable(25) and DoorUnlockedOrUnlockable(27) then
        RevealWeaponRoom()
      end
    end
  end
}

ZP07Bedroom01 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      if IsDoorLockedPC(GetDoorScript(22)) == false then
        RevealBedRoom()
      end
    end
  end
}

ZP07Bedroom02 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      if DoorUnlockedOrUnlockable(23) and DoorUnlockedOrUnlockable(24) then
        RevealBedRoom()
      end
    end
  end
}

ZP09Staircase01 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      RevealStaircase()
    end
  end
}

ZP09Staircase02 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      RevealStaircase()
    end
  end
}